@@ -5,7 +5,7 @@ from foundry.game.gfx.drawable import apply_selection_overlay
 from foundry.game.gfx.drawable.Block import Block
 from foundry.game.gfx.GraphicsSet import GraphicsSet
 from foundry.game.gfx.objects.ObjectLike import ObjectLike
-from foundry.game.gfx.Palette import NESPalette, PaletteGroup
+from foundry.game.gfx.Palette import PaletteGroup
 from foundry.game.ObjectDefinitions import (
     enemy_handle_x,
     enemy_handle_x2,
@@ -39,8 +39,6 @@ class EnemyObject(ObjectLike):
 
         self.object_set = ObjectSet(ENEMY_ITEM_OBJECT_SET)
 
-        self.bg_color = NESPalette[palette_group[0][0]]
-
         self.png_data = png_data
 
         self.selected = False
