@@ -1,12 +1,14 @@
+from json import loads
 from typing import Optional
 
 from PySide6.QtCore import Signal, SignalInstance
-from PySide6.QtWidgets import QFormLayout, QSizePolicy, QWidget
+from PySide6.QtWidgets import QSizePolicy, QWidget
 
+from foundry import spinner_panel_flags_path
 from foundry.game.gfx.objects.LevelObject import LevelObject
 from foundry.game.gfx.objects.ObjectLike import ObjectLike
 from foundry.game.level.LevelRef import LevelRef
-from foundry.gui.Spinner import Spinner
+from foundry.gui.util import setup_description, setup_layout
 
 MAX_DOMAIN = 0x07
 MAX_TYPE = 0xFF
@@ -27,41 +29,10 @@ class SpinnerPanel(QWidget):
         self.level_ref = level_ref
         self.level_ref.data_changed.connect(self.update)
 
-        self.spin_domain = Spinner(self, maximum=MAX_DOMAIN)
-        self.spin_domain.setEnabled(False)
-        self.spin_domain.valueChanged.connect(self.object_change.emit)
-
-        self.spin_type = Spinner(self, maximum=MAX_TYPE)
-        self.spin_type.setEnabled(False)
-        self.spin_type.valueChanged.connect(self.object_change.emit)
-
-        self.spin_length = Spinner(self, maximum=MAX_LENGTH)
-        self.spin_length.setEnabled(False)
-        self.spin_length.valueChanged.connect(self.object_change.emit)
-
-        spinner_layout = QFormLayout()
-        spinner_layout.addRow("Bank/Domain:", self.spin_domain)
-        spinner_layout.addRow("Index:", self.spin_type)
-        spinner_layout.addRow("Length:", self.spin_length)
-
-        self.setLayout(spinner_layout)
-
-        self.setWhatsThis(
-            "<b>Spinner Panel</b><br/>"
-            "The Spinner Panel gives raw byte access to objects for advanced users. The values are shown "
-            "in hexadecimal notation.<br/>"
-            "Level objects and enemies/items are categorized using domains and indexes. Which domain an "
-            "object is in, doesn't hold much information about the object, if at all.<br/>"
-            "As for the index, the only important information is, that all objects from 0x00 - 0x0F can "
-            "not be resized. "
-            "They have fixed dimensions, like the background bushes in Level 1-1.<br/>"
-            "All other objects have 16 different iterations, meaning 0x10 - 0x1F, for example, is one "
-            "object, with 16 different sizes, going from smallest to largest. In what way these objects "
-            "expand, depends on their particular expansion type.<br/>"
-            "Some '4-byte' objects can expand in a second way, since they have an additional byte "
-            "holding that information. For example a platform, which can be sized vertically using the "
-            "index and horizontally using the 4th byte."
-        )
+        with open(spinner_panel_flags_path, "r") as data:
+            flags = loads(data.read())
+        setup_layout(self, flags)
+        setup_description(self, flags)
 
     def update(self):
         if len(self.level_ref.selected_objects) == 1:
